<?php
/**
 * Viper2 Framework
 *
 * @author fish <fish@archa.cz>
 * @package Viper2
 * @version 0.1
 *
 */

class Viper2 {
  /**
   * variable $useSmarty
   * @static boolean provazat se smarty ?
   */
  static $useSmarty = FALSE;
  /**
   * static $smarty
   * @static object odkaz na smarty
   */
  static $smarty = NULL;
  /**
   * static $smartyCache
   * @static integer nastaveni smarty cache
   */
  static $smartyCache = 0;
  /**
   * static $dbh
   * @static object database handle
   */
  static $dbh = NULL;
  /**
   * static $renderFinished
   * @static boolean probehl render layoutu ?
   */
  static $renderFinished = FALSE;
  /**
   * static $variables
   * @static array pole globalnich promennych viperu
   */
  protected static $variables = FALSE;

  /**
   * variable $addShortLinks
   * @var boolean pridavat variantu linku bez index.htm
   */
  static $addShortLinks = FALSE;

  /**
   * variable $addVeryShortLinks
   * @var boolean pridavat variantu linku bez index.htm a zaverecneho /
   */
  static $addVeryShortLinks = FALSE;
  
  /**
   * static $router
   * @static object objekt routeru
   */
  static $router = NULL;
  /**
   * static $layout
   * @static string zobrazovany layout
   */
  protected static $layout = "index.tpl";
  /**
   * static $loadedPlugins
   * @static array seznam zaregistrovanych pluginu. Prvni pole ma index prioritu zpracovani, druhe je cislovano od nuly
   */
  protected static $loadedPlugins = array();
  /**
   * static $activePlugins
   * @static array - pole indexovano jmenem tridy pluginu obsahujici reference na nactene instance
   */
  protected static $activePlugins = array();
  
  /**
   * static $layoutPostProcess
   * @static array funkce pro postprocessing vystupu
   */
  static $layoutPostProcess = NULL;
  
  /**
   * static $forceCanonicalURI
   * @static boolean vynuti jednoznacnost URI
   */
  static $forceCanonicalURI = TRUE;
  

  // {{{ init
  /**
   * nacte dostupny pluginy
   *
   * @return void
   */
  static function init($path, $cfgPath){
    if (!file_exists($path) || !is_dir($path)) {
      throw new Exception("Viper2 plugin directory not found: ".$path."");
    }
    
    self::openPluginDir($path, $cfgPath);

    ksort(self::$loadedPlugins);
    
    // nacteni konfigurace atd.
    foreach (self::$activePlugins as $pluginName=>$pluginObj) {
      //nacteni konfigurace pokud existuje
      if (file_exists($cfgPath."/".$pluginObj->getName().".inc.php") && is_file($cfgPath."/".$pluginObj->getName().".inc.php")) {
        // echo $path."/".$file.": ".get_class($pluginObj)."->".$pluginObj->getName()._nl;
        require $cfgPath."/".$pluginObj->getName().".inc.php";
      }
      
      // znovu kontrola jestli neni plugin zakazany z konfigu
      if ($pluginObj->isEnabled()) {
        //nastaveni smarty bloku
        if (isset(self::$smarty)) {
          $reflect = new ReflectionClass(get_class($pluginObj));
          $methods = $reflect->getMethods();
          foreach ($methods as $method) {
            if ($method->isPublic() && preg_match("/^smarty_(.+)$/", $method->getName(), $mtmp)) {
              self::$smarty->register_function($pluginObj->getName()."_".$mtmp[1], array($pluginObj, $method->getName()));
            }
          }
          unset($methods);
          unset($reflect);
        }
      } else {
        self::unregister($pluginObj->getName());
        unset($pluginObj);
      }
    }
    

    // if(isset(self::$smarty)){
    //   self::$smarty->assign_by_ref("Viper2",self::$VAR);
    // }
    if (self::$useSmarty) {
      self::$smarty->register_function("link", array("Viper2","makeLinkSmarty"));
      self::$smarty->register_function("V2Render", array("Viper2","processRender"));
      self::$smarty->assign_by_ref("viper", self::$variables);
      self::$smarty->assign_by_ref("router", self::$router);
    }
  }
  // }}} init
  
  protected function openPluginDir($path, $cfgPath) {
    $pluginFiles = array();
    $pluginDirs = array();
    if ($dh = opendir($path)) {
      while(($file = readdir($dh)) !== false){
        if (is_file($path."/".$file)) {
          $pluginFiles[] = $file;
        }
        if (is_dir($path."/".$file) && !preg_match("/^\./", $file)) {
          $pluginDirs[] = $file;
        }
      }
      closedir($dh);
      
      sort($pluginFiles);
      sort($pluginDirs);
      
      // nejdriv nactu pluginy v aktualnim adresari
      
      foreach($pluginFiles as $file){
        if (is_file($path."/".$file) && preg_match("/^([^\/\.]+)\.php$/",$file,$cname)) {
          $className=$cname[1];
          
          require_once $path."/".$file;
          
          $pluginObj = new $className();

          // z podminky jsem zrusil ignorovani pluginu s vychozim enabled=FALSE
          // pro vetsi moznosti rizeni pres config (viz. Memcached)
          if (/* $pluginObj->isEnabled() && */ self::register($pluginObj)) {
            // automaticke vytvoreni cesty k template z nazvu classy pluginu
            // if ($pluginObj->getTplPath()===NULL) {
            //   $pluginObj->setTplPath(preg_replace("/_/", "/", get_class($pluginObj))."/");
            // }
            
            // tady puvodne bylo nacteni konfigurace a registrace smarty funkcni
            // kvuli extendovani doslo k presunuti za nacteni vsech pluginu
            
          } else {
            unset($pluginObj);
          }
        }
      }
      
      // pak rekurzivne projdu dalsi adresare
      foreach($pluginDirs as $dir){
        self::openPluginDir($path."/".$dir, $cfgPath);
      }
      
    }
  }
  
  
  static function makeLinkSmarty($params, &$smarty) {
    echo self::$router->basePage().self::makeLink($params);
  }

  static function makeLink($params) {
    return(self::$router->makeLink($params));   
  }

  // {{{ register
  /**
   * zaregistruje plugin do frameworku
   *
   * @param object &$plugin objekt pluginu
   * @return boolean
   */
  static function register($plugin){
    if ($plugin instanceof Viper2Plugin) {
      if ($plugin->getName() == "Viper2Plugin") {
        throw new Exception("Trying to register plugin ".get_class($plugin)." with default name Viper2Plugin");
      }
      // if ($plugin->getUri() === NULL) {
      //   throw new Exception("Plugin ".get_class($plugin)." has no \$pluginUri defined");
      // }
      if (isset(self::$activePlugins[$plugin->getName()])) {
        // pokus uz pod danym nazvem byl jiny plugin, odregistruju ho
        self::unregister($plugin->getName());
      }
      $plugin->setDbh(self::$dbh);
      self::$loadedPlugins[$plugin->getPriority()][$plugin->getName()] = $plugin;
      self::$activePlugins[$plugin->getName()] = $plugin;
    } else {
      return(FALSE);
    }
    return(TRUE);
  }
  // }}} register

  // {{{ unregister
  /**
   * odregistruje plugin z frameworku
   *
   * @param string nazev pluginu
   * @return boolean
   */
  static function unregister($pluginName){
    foreach (self::$loadedPlugins as &$pluginGroup) {
      foreach ($pluginGroup as $pluginId=>$plugin) {
        if ($pluginId==$pluginName) {
          unset($pluginGroup[$pluginId]);
        }
      }
    }
    unset(self::$activePlugins[$pluginName]);
    return(TRUE);
  }
  // }}} unregister

  // {{{ initRouter
  /**
   * inicializace routeru
   *
   * @param object $router objekt routeru
   * @return void
   */
  static function initRouter($router){
    self::$router = $router;
    
    foreach (self::$activePlugins as $plugin) {
      $pluginUri = $plugin->getUri();
      if ($plugin->isProcessing() && isset($pluginUri)) {
        $reflect = new ReflectionClass(get_class($plugin));
        $methods = $reflect->getMethods();
        foreach ($methods as $method) {
          if (preg_match("/(?<actionType>action|show|render)(?<actionName>.+)/", $method->getName(), $match)) {
            $action = strtolower($match["actionName"]);
            // globalni akce se neregistruje jako handler, protoze se vola vzdy
            if ($action!="global") {
              $methodComment = $method->getDocComment();
              // echo get_class($plugin)."->".$method->getName()." : ".$action._nl;
              // echo $methodComment._nl;
              $layout = "default";
              $masks = "";
              if (preg_match("/@layout:?\s+\"([^\"]+)\"/", $methodComment, $match)) {
                $layout = $match[1];
              }
              // echo "layout: ".$layout._nl;
              if (preg_match_all("/@mask:?\s+([^\r\n]+)/", $methodComment, $match)) {
                $masks = $match[1];
              } else {
                // jedna prazdna maska pro default linky
                $masks[] = "";
              }
              // pre_print($masks);
              // echo $plugin->getName()."::".$action."@".$layout._nl;
              
              
              foreach ($masks as $mask){
                $linkDirection = "both";
                // naslednou kontrolu mozno prepsat na substr kvuli optimalizaci vykonu
                if (preg_match("/^([<>])(.+)$/", $mask, $match)) {
                  $mask = $match[2];
                  if ($match[1]=="<") {
                    $linkDirection = "out";
                  } else {
                    $linkDirection = "in";
                  }
                }
                $params = array();
                if (preg_match_all("/\?<([^\>]+)>/", $mask, $match)) {
                  $params = $match[1]; 
                }
                $link = self::$router->buildLink($plugin->getUri(), $action, $mask);
                // var_dump($link);
                // echo "layout: ".$layout._nl;
                if (is_array($link)) {
                  foreach ($link as $subLink) {
                    self::$router->setRoute($subLink, $plugin->getName(), $action, $layout, $mask, $params, self::$addShortLinks, self::$addVeryShortLinks, $linkDirection);
                  }
                } else {
                  self::$router->setRoute($link, $plugin->getName(), $action, $layout, $mask, $params, self::$addShortLinks, self::$addVeryShortLinks, $linkDirection);
                }
              }
            }

          }
        }
        unset($methods);
        unset($reflect);
      }
    }
    
    self::$router->setDirection();
    $layout = self::$router->getLayout();
    if ($layout!="default") {
      self::$layout = $layout;
    }
    if (self::$useSmarty) {
      $params = self::$router->getParams();
      self::$smarty->assign("viperParams", $params);
    }
    
    // v2("Msgbox", "add", self::$router->getPlugin()."::".self::$router->getAction()."(".join(",", self::$router->getParams()).")");
    
    
    // vynuceni jednotneho URL
    if (self::$forceCanonicalURI && $_SERVER["REQUEST_METHOD"]=="GET" && self::$router->getPlugin()."::".self::$router->getAction()!="Default::404") {
      $actLinkInfo = self::$router->getParams();
      $actLinkInfo["to"] = self::$router->getPlugin()."::".self::$router->getAction();
      $canonicalURI = self::makeLink($actLinkInfo);
      // pre_print($actLinkInfo);
      // echo $canonicalURI._nl;
      // echo self::$router->uri()._nl;
      // SafeExit();
      if ($canonicalURI!=self::$router->uri()) {
        $getParams = "";
        if (count($_GET)) {
          foreach ($_GET as $getName=>$getVar) {
            $getParams.=($getParams==""?"?":"&").$getName."=".rawurlencode($getVar);
          }
        }
        self::$router->redirectTo(self::$router->basePage().$canonicalURI.$getParams, TRUE);
        // v2("Msgbox", "add", $canonicalURI);
      }
    }
    
  }
  // }}} initRouter
  
  // {{{ processActions
  /**
   * zpracovani akci
   *
   * @return void
   */
  static function processActions(){
    $pluginName = self::$router->getPlugin();
    $action = self::$router->getAction();
    $functionName = "action".$action;
    $params = self::$router->getParams();

    // nejdriv zpracuju vsechny globalni akce
    self::processGlobal("action", $params);
    
    if (!isset(self::$activePlugins[$pluginName])) {
      return(FALSE);
      // throw new Exception("Route direction to undefined plugin ".$pluginName);
    }
    if (!method_exists(self::$activePlugins[$pluginName],$functionName)) {
      return(FALSE);
      // throw new Exception("Call of undefined function ".$pluginName."->".$functionName);
    }
    
    call_user_func_array(array(self::$activePlugins[$pluginName], $functionName), array($params));
  }
  // }}} processActions
  
  // {{{ processShow
  /**
   * zobrazeni mimo layout
   *
   * @return void
   */
  static function processShow(){
    $pluginName = self::$router->getPlugin();
    $action = self::$router->getAction();
    $functionName = "show".$action;
    $params = self::$router->getParams();

    // nejdriv zpracuju vsechny globalni show
    self::processGlobal("show", $params);
    
    if (!isset(self::$activePlugins[$pluginName])) {
      return(FALSE);
    }
    if (!method_exists(self::$activePlugins[$pluginName],$functionName)) {
      return(FALSE);
    }
   
    call_user_func_array(array(self::$activePlugins[$pluginName], $functionName), array($params));
  }
  // }}} processShow
  
  // {{{ processRender
  /**
   * zobrazeni dynamicke casti layoutu
   *
   * @return void
   */
  static function processRender($layoutParams=array(), &$smarty){
    $pluginName = self::$router->getPlugin();
    $action = self::$router->getAction();
    $params = self::$router->getParams();
    
    // nejdriv zpracuju vsechny globalni rendery
    self::processGlobal("render", array_merge($params, $layoutParams), $smarty);
    
    self::render($pluginName, $action, array_merge($params, $layoutParams));
        
    self::$renderFinished = TRUE;
  }
  // }}} processRender
  
  /**
  * vyrenderuje akci pluginu
  *
  */
  static function render($pluginName, $action, $params=array()) {
    if (!isset(self::$activePlugins[$pluginName])) {
      return(FALSE);
    }
    $functionName = "render".$action;
    if (method_exists(self::$activePlugins[$pluginName],$functionName)) {
      call_user_func_array(array(self::$activePlugins[$pluginName], $functionName), array($params, &Viper2::$smarty));
      // aktivace CSS
      call_user_func(array(self::$activePlugins[$pluginName], "useCSS"));
    }
  }

  // {{{ renderLayout
  /**
   * zobrazeni layoutu
   *
   * @return void
   */
  static function renderLayout(){
    if (self::$useSmarty && isset(self::$layout)) {
      $layoutContent = self::$smarty->fetch(self::$layout);
      if (isset(self::$layoutPostProcess) && 
          ((is_string(self::$layoutPostProcess) && function_exists(self::$layoutPostProcess)) ||
           (is_array(self::$layoutPostProcess) && method_exists(self::$layoutPostProcess[0], self::$layoutPostProcess[1]))
          )
         ) {
        $layoutContent = call_user_func_array(self::$layoutPostProcess, array($layoutContent));
      }
      echo $layoutContent;
    }
  }
  // }}} renderLayout
  
  // {{{ getActivePluginNames
  /**
   * nazvy aktivnich pluginu
   *
   * @return array
   */
  public function getActivePluginNames() {
    return(array_keys(self::$activePlugins));
  }
  // }}} getActivePluginNames

  // {{{ getVar
  /**
   * promenna
   *
   * @return mixed
   */
  public function getVar($varName) {
    if (!isset(self::$variables[$varName])) {
      return(NULL);
    } else {
      return(self::$variables[$varName]);
    }
  }
  // }}} getVar

  // {{{ setVar
  /**
   * promenna
   *
   * @return void
   */
  public function setVar($varName, $value) {
    self::$variables[$varName] = $value;
  }
  // }}} setVar

  
  // {{{ call
  /**
   * zavola funkci pluginu
   *
   * @return array
   */
  public function call() {
    $params = func_get_args();
    if (count($params)<2) {
      throw new Exception("Not enough parameters for plugin call");
    }

    $pluginName = array_shift($params); 
    $functionName = array_shift($params);
    if (!isset(self::$activePlugins[$pluginName])) {
      throw new Exception("Call of undefined plugin ".$pluginName);
    }
    if (!method_exists(self::$activePlugins[$pluginName],$functionName)) {
      throw new Exception("Call of undefined function ".$pluginName."->".$functionName);
    }
    
    return(call_user_func_array(array(self::$activePlugins[$pluginName], $functionName), $params));

  }
  // }}} call
  
  // {{{ setLayout
  /**
   * nastavi cilovy layout
   *
   * @return void
   */
  public function setLayout($layout) {
    self::$layout = $layout;
  }
  // }}} setLayout


  // {{{ processGlobal
  /**
   * zpracovani globalni sekce aktivnich pluginu
   *
   * @return void
   */
  static function processGlobal($type, $params=NULL, &$s=NULL) {
    if ($type!="action" && $type!="show" && $type!="render") {
      throw new Exception("Trying to process unknown type '".$type."'");
    }
    
    $methodName = $type."Global";
    foreach(self::$loadedPlugins as $prior=>$pluginGroup){
      foreach($pluginGroup as $plugin){
        if ($plugin->isProcessing() && method_exists($plugin,$methodName)) {
        // echo $plugin->getName()."->".$methodName._nl;
        // echo "#".get_class($s)._nl;;
          if ($s instanceof Smarty) {
            call_user_func_array(array($plugin, $methodName), array($params, $s));
          } else {
            call_user_func_array(array($plugin, $methodName), $params);
          }
        }
      }
    }
  }
  // }}} processGlobal

  // {{{ processCSS
  /**
   * vygeneruje seznam pouzitych css
   *
   * @return string
   */
  static function processCSS(){
    $cssList = "";
    $Included = array();
    if (self::$renderFinished && self::$useSmarty) {
      foreach(self::$activePlugins as $plugin){
/*
        $cssLink = $plugin->getCSS();
        if($plugin->isCSSUsed() && !empty($cssLink)){
          $cssList.="<link rel=\"stylesheet\" href=\"".self::$router->basePage().$plugin->getCSS().(!is_null($plugin->getCSSStamp())?"?".$plugin->getCSSStamp():"")."\" type=\"text/css\" /> <!-- ".$plugin->getName()." -->\n";
*/
        $usedCSS = $plugin->getUsedCSS();
        foreach ($usedCSS as $cssBlock=> $CSSParams) {
          $links = $CSSParams['src'];
          if (!is_array($links)) {
            $links=array($links);
          }
          foreach ($links as $link) {
            if (!in_array($link, $Included)) {
              $cssList.="<link rel=\"stylesheet\" href=\"".(preg_match('/^http/', $link)?'':self::$router->basePage()).$link.(isset($CSSParams['stamp'])?(preg_match('/\?/', $link)?'&':"?").$CSSParams['stamp']:"")."\" type=\"text/css\" /> <!-- ".$plugin->getName()." - ".$cssBlock." -->\n";
              $Included[]=$link;
            }
          }
        }
      }
    }
    return($cssList);
  }
  // }}} processCSS
  

  // {{{ processJS
  /**
   * vygeneruje seznam pouzitych js
   *
   * @return string
   */
  static function processJS(){
    $jsList = "";

    $Included = array();
    if(self::$renderFinished && isset(self::$useSmarty)){
      foreach (self::$activePlugins as $plugin) {
        $usedJs = $plugin->getUsedJS();
        foreach ($usedJs as $JsParams) {
          $Scripts = $JsParams['src'];
          if (!is_array($Scripts)) {
            $Scripts=array($Scripts);
          }
          foreach ($Scripts as $Script) {
            if (!in_array($Script, $Included)) {
              $jsList.='<script language="javascript" type="text/javascript" src="'.(preg_match('/^http/', $Script)?'':_BASE_PAGE).$Script.(isset($JsParams['stamp'])?(preg_match('/\?/', $Script)?'&':"?").$JsParams['stamp']:"").'"></script>'."\n";
              $Included[]=$Script;
            }
          }
        }
      }
    }
    return($jsList);
  }
  // }}} processJS
  
  // {{{ getParam
  /**
   * vrati obsah jednoho parametru routeru pokud existuje
   *
   * @return mixed
   */
  public function getParam($param) {
    return(self::$router->getParam($param));
  }
  // }}} getParam
  
  static function dumpVars() {
    pre_print(self::$variables);
  }
  
  static function getPlugin($pluginName) {
    if (!isset(self::$activePlugins[$pluginName])) {
      throw new Exception("Call of undefined plugin ".$pluginName);
    }
    return(self::$activePlugins[$pluginName]);
  }

  static function hasPlugin($pluginName) {
    return(in_array($pluginName, Viper2::getActivePluginNames()));
  }
  
  // {{{ getKeywords
  /**
   * vygeneruje seznam klicovych slov
   *
   * @return string
   */
  static function getKeywords(){
    $keywords = array();
    if(self::$renderFinished && isset(self::$useSmarty)){
      foreach (self::$activePlugins as $plugin) {
        $keywords = array_merge($keywords, $plugin->getKeywords());
      }
      $keywords = array_unique($keywords);
    }
    return implode(', ', $keywords);
  }
  // }}} getKeywords
  
}
  // {{{ v2
  /**
   * zkratka pro Viper2::call()
   *
   * @return array
   */

  function v2() {
    $params = func_get_args();
    return(call_user_func_array(array("Viper2","call"), $params));
  }
  // }}} v2
  
  final class v2 {
    /**
     * variable $instance
     * @var object instance classy
     */
    protected static $instance = NULL;

    public function getInstance() {
      if (self::$instance === NULL) {
        self::$instance = new v2;
      }
      return(self::$instance);
    }
    
    function __get($name) {
      return(Viper2::getPlugin($name));
    }
/*    
    function __callStatic($name, $args) {
      if ($name=="get") {
        $pluginName = $args[0];
        return(Viper2::getPlugin($pluginName));
      }
    }
*/
  }
  // $v2 = v2::getInstance();
  
