<?php

/**
 * Viper2 Framework
 *
 * @author fish <fish@archa.cz>
 * @package Viper2
 * @version 0.1
 *
 */
/**
 * kostra Viper2 pluginu
 * @package Viper2Plugin
 * @author fish <fish@archa.cz>
 * @version 0.1
 *
 *
 */

 class Viper2Plugin {
  /**
   * variable $pluginName
   * @var string unikatni nazev pluginu
   */
  protected $pluginName = "Viper2Plugin";
  /**
   * variable $tplPath
   * @var string cesta k template pluginu
   */
  protected $tplPath = "";
  /**
   * variable $pluginUri
   * @var string cast url pod kterou bude obsah pluginu (klidne i /dir1/dir2...)  
   */
  protected $pluginUri = NULL;
  /**
   * variable $enabled
   * @var boolean je plugin povoleny ?
   */
  protected $enabled = TRUE;
  /**
   * variable processing
   * @var boolean zpracovat akce pluginu ?
   */
  protected $processing = TRUE;
  /**
   * variable $dbh
   * @var object database handle
   */
  protected $dbh = NULL;
  /**
   * variable $priority
   * @var boolean poradi pluginu pri zpracovani ve frameworku
   */
  protected $priority = 0;

  /**
   * variable $cssLink
   * @var mixed odkaz(y) na css
   */
  protected $cssLink = NULL;
  /**
   * variable $cssStamp
   * @var string stamp css souboru
   */
  protected $cssStamp = NULL;
  /**
   * variable $cssUsed
   * @var boolean byl plugin pouzit a ma se nacist CSS ?
   */
  protected $cssUsed = NULL;
  /**
   * variable $jsList
   * @var array(<jsBlock> => 'include'=>boolean, 'src'=>array(<jsFilePath>, ... )[, 'stamp'=>string]), ... )
   */
  protected $jsList = array();
  /**  
   * variable $cssList
   * @var array(<cssBlock> => 'include'=>boolean, 'src'=>array(<cssFilePath>, ... )[, 'stamp'=>string]), ... )
   * 
   * nova varianta cssUsed,cssLink a cssStamp - pri vyplneni teto promenne jsou predchozi 3 promenne ignorovany      
   */
  protected $cssList = array();
  /**
   * protected $keywords
   * @var array of string - pole klicovych slov
   */        
  protected $keywords = array();

  /**
   * variable $uaMap
   * @var array prirazeni uzivatelskych akci k UA promennym
   */
  protected $uaMap = array();
  /**
   * variable $rights
   * @var array opravneni uzivatele
   */
  protected $rights = array();
  
  /**
   * variable $tblPrefix
   * @var string prefix nazvu tabulek (muze byt vcetne databaze)
   */
  protected $tblPrefix = "";
  /**
   * variable $tblPrefixLocal
   * @var string prefix nazvu tabulek pro zapis (muze byt vcetne databaze)
   */
  protected $tblPrefixLocal = "";

  
  // {{{ getName
  /**
   * nazev pluginu
   *
   * @return string
   */
  public function getName() {
    return($this->pluginName);
  }
  // }}} getName
  
  // {{{ getTplPath
  /**
   * cesta k template
   *
   * @return string
   */
  public function getTplPath() {
    return($this->tplPath);
  }
  // }}} getTplPath

  // {{{ setTplPath
  /**
   * cesta k template
   *
   * @return void
   */
  public function setTplPath($path) {
    $this->tplPath = $path;
  }
  // }}} setTplPath

  // {{{ getCSS
  /**
   * soubor CSS
   *
   * @return string
   */
  public function getCSS() {
    return($this->cssLink);
  }
  // }}} getCSS

  // {{{ getCSSStamp
  /**
   * stamp CSS
   *
   * @return string
   */
  public function getCSSStamp() {
    return($this->cssStamp);
  }
  // }}} getCSSStamp

  // {{{ setCSSStamp
  /**
   * stamp CSS
   *
   * @return string
   */
  public function setCSSStamp($stamp=NULL) {
    if (!isset($stamp)) {
      $stamp = date("YmhdHi");
    }
    $this->cssStamp = $stamp;
  }
  // }}} setCSSStamp


  // {{{ isCSSUsed
  /**
   * byl plugin pouzit ?
   *
   * @return string
   */
  public function isCSSUsed() {
    if (!empty($this->cssList)) {
      foreach ($this->cssList as $blockName=>$blockInfo) {
        if (!empty($blockInfo['include'])) {
          return true;
        }
      }
      return false;
    } else {
      return($this->CSSUsed);
    }
  }
  // }}} isCSSUsed

  // {{{ useCSS
  /**
   * aktivovat CSS
   * 
   * @param @optional $cssBlock - default 'plugin'
   * @return void
   */
  public function useCSS($cssBlock='plugin') {
    if (!isset($this->cssList[$cssBlock])) {
      $this->CSSUsed = TRUE;
    } else {
      return $this->cssList[$cssBlock]['include'] = true;
    }
  }
  // }}} useCSS

  // {{{ getUsedCSS
  /**
   * vrati seznam pouzitych CSS
   *     
   * @return array(<cssBlockName> => <cssParams - see $this->cssList>, ...)
   */     
  public function getUsedCSS() {
    if (empty($this->cssList)) {
      if ($this->CSSUsed && !empty($this->cssLink)) {
        return array('plugin'=>array('include'=>1, 'src'=>array($this->cssLink), 'stamp'=>$this->cssStamp));
      } else {
        return array();
      }
    } else {
      $usedCSS = array();
      foreach ($this->cssList as $cssBlockName => $cssParams) {
        if (!empty($cssParams['include'])) {
          $usedCSS[$cssBlockName] = $cssParams;
        }
      }
      return $usedCSS;
    }
  }
  // }}} getUsedCSS

  // {{{ useJS
  /**
   * aktivovat javascript
   * 
   * @param @optional $jsBlock - default 'plugin'
   *       
   * @return boolean (blok je definovan a aktivovan = true, neni definovan = false)
   */           
  public function useJS($jsBlock = 'plugin') {
    if (!isset($this->jsList[$jsBlock])) {
      return false;
    }
    return $this->jsList[$jsBlock]['include'] = true;
  }
  // }}} useJS

  // {{{ getUsedJS
  /**
   * vrati vsechny pouzite JS bloky
   * 
   * @return array(<jsBlockName> => <jsParams - see $this->jsList>, ...)
   */
  public function getUsedJS() {
    $usedJs = array();
    foreach ($this->jsList as $jsBlockName => $jsParams) {
      if (!empty($jsParams['include'])) {
        $usedJs[$jsBlockName] = $jsParams;
      }
    }
    return $usedJs;
  }
  // }}} getUsedJS


  // {{{ enable
  /**
   * nastavi plugin jako povoleny
   *
   * @return void
   */
  public function enable($state=TRUE) {
    $this->enabled = $state;
  }
  // }}} enable

  // {{{ isEnabled
  /**
   * je plugin povolen ?
   *
   * @return boolean
   */
  public function isEnabled() {
    return($this->enabled);
  }
  // }}} isEnabled
  
  // {{{ setProcessing
  /**
   * nastavi zpracovani pluginu
   *
   * @return void
   */
  public function setProcessing($processing) {
    $this->processing = $processing;
  }
  // }}} setProcessing

  // {{{ isProcessing
  /**
   * ma se plugin zpracovavat ?
   *
   * @return boolean
   */
  public function isProcessing() {
    return($this->processing);
  }
  // }}} isProcessing

  // {{{ setPriority
  /**
   * priorita pluginu
   *
   * @return void
   */
  public function setPriority($priority) {
    $this->priority=$priority;
  }
  // }}} setPriority

  // {{{ getPriority
  /**
   * priorita pluginu
   *
   * @return int
   */
  public function getPriority() {
    return($this->priority);
  }
  // }}} getPriority
  
  // {{{ getUri
  /**
   * uri pluginu
   *
   * @return string
   */
  public function getUri() {
    return($this->pluginUri);
  }
  // }}} getUri

  // {{{ setUri
  /**
   * uri pluginu
   *
   * @return void
   */
  public function setUri($uri) {
    $this->pluginUri = $uri;
  }
  // }}} setUri
  

  // {{{ setDbh
  /**
   * nastaveni handle k databazi
   *
   * @return void
   */
  public function setDbh($dbh) {
    if (!is_a($dbh,"MDB2_Driver_mysql")) {
      throw new Exception('Passed $dbh is not correct database handle'); 
    }
    $this->dbh = $dbh;
  }
  // }}} setDbh
  
  
  // {{{ actionDefault
  /**
   * funkce ktera se zavola pred vizualnim vystupem (zapisy do db, redirecty atp.)
   *
   * klicove slovo nazvu akce je Global - takova akce se provede vzdy
   * nezavisle na cilovem pluginu
   *
   * @return void
   */
  public function actionDefault($params=array()) {
  }
  // }}} actionDefault

  // {{{ showDefault
  /**
   * funkce pro zobrazeni obsahu pokud se nebude pouzivat vystup uvnitr layoutu
   *
   * klicove slovo nazvu akce je Global - takova akce se provede vzdy
   * nezavisle na cilovem pluginu
   *
   * @return void
   */
  public function showDefault($params=array()) {
  }
  // }}} showDefault

  // {{{ renderDefault
  /**
   * zobrazeni obsahu v ramci layoutu
   *
   * klicove slovo nazvu akce je Global - takova akce se provede vzdy
   * nezavisle na cilovem pluginu
   *
   * @param array $params parametry smarty bloku
   * @param object &$s smarty objekt
   * @return void
   */
  public function renderDefault($params, &$s) {
    if(Viper2::$useSmarty){
    }
  }
  // }}} renderDefault

  // {{{ setUaMap
  /**
   * nastavi tabulku prirazeni prav
   *
   * @return void
   */
  public function setUaMap($map) {
    $this->uaMap = $map;
  }
  // }}} setUaMap
  
  // {{{ setRight
  /**
   * nastavi pravo
   *
   * @return void
   */
  public function setRight($rightName, $value) {
    $this->rights[$rightName] = $value;
  }
  // }}} setRight

  // {{{ canDo
  /**
   * ma povolene pravo ?
   *
   * @return mixed
   */
  public function canDo($rightName) {
    if (!isset($this->rights[$rightName]) || $this->rights[$rightName]==0) {
      return(FALSE);
    } else {
      return(TRUE);
    }
  }
  // }}} canDo
  
  // {{{ loadRights
  /**
   * nacteni prav pluginu
   *
   * @return void
   */
  function loadRights() {
    $flogin = Viper2::getVar("flogin");
    if (!is_null($flogin)) {
      foreach ($this->uaMap as $rightName=>$varName) {
        $this->setRight($rightName, $flogin->canDoAction(Viper2::getVar("loginUid"), $varName));
        if (Viper2::$useSmarty) {
          Viper2::$smarty->assign(strtoupper($this->getName())."_".$rightName, $this->canDo($rightName));
        }
      }
    }
  }
  // }}} loadRights
  
  // {{{ setTblPrefix
  /**
   * nastavi prefix tabulek
   *
   * @return void
   */
  public function setTblPrefix($prefix) {
    $this->tblPrefix = $prefix;
  }
  // }}} setTblPrefix

  // {{{ setTblPrefixLocal
  /**
   * nastavi prefix tabulek
   *
   * @return void
   */
  public function setTblPrefixLocal($prefix) {
    $this->tblPrefixLocal = $prefix;
  }
  // }}} setTblPrefixLocal

  // {{{ getTblPrefix
  /**
   * zjisti prefix tabulek
   *
   * @return void
   */
  public function getTblPrefix() {
    return $this->tblPrefix;
  }
  // }}} getTblPrefix

  // {{{ getTblPrefixLocal
  /**
   * zjisti prefix tabulek
   *
   * @return void
   */
  public function getTblPrefixLocal($prefix) {
    return $this->tblPrefixLocal;
  }
  // }}} getTblPrefixLocal

  // {{{ setCfg
  /**
   * nastaveni konfiguracni hodnoty
   *
   * @return void
   */
  public function setCfg($var, $value) {
    $varName = "cfg".$var;
    if (!property_exists($this, $varName)) {
      throw new Exception("Trying to write undefined config property ".$this->getName()."->cfg".$var."");
    } else {
      $this->$varName = $value;
    }
  }
  // }}} setCfg

  // {{{ getCfg
  /**
   * cteni konfiguracni hodnoty
   *
   * @return mixed
   */
  public function getCfg($var) {
    $varName = "cfg".$var;
    if (!property_exists($this, $varName)) {
      throw new Exception("Trying to read undefined config property ".$this->getName()."->cfg".$var."");
    } else {
      return($this->$varName);
    }
  }
  // }}} getCfg

  // {{{ addKeywords
  /**
   * prida klicova slova pro vypsani v headeru stranky
   *
   * @param string | array of string
   */
  protected function addKeywords($keywords) {
    if (empty($keywords)) {
      return;
    }
    if (is_array($keywords)) {
      $this->keywords = array_merge($this->keywords, $keywords); 
    } else {
      // prisel mi string - pouzijeme vsechna slova delsi nez 3 znaky
      $parts = explode(' ', preg_replace('/[^-\p{L}\d]/u', ' ', $keywords));
      foreach ($parts as $word) {
        if (mb_strlen($word)>3) {
          $this->keywords[] = $word;
        }
      }
    }
    $this->keywords = array_unique($this->keywords);
  }
  // }}} addKeywords
  
  // {{{ getKeywords
  /**
   * funkce vrati nastavena klicova slova
   *
   * @return array
   */
  public function getKeywords() {
    return $this->keywords;
  }
  // }}} addKeywords           

}
?>