<?php

/**
 * Viper2 Framework
 *
 * @author fish <fish@archa.cz>
 * @package Viper2
 * @version 0.1
 *
 */
/**
 * parser URL a router
 * @package Viper2Router
 * @author fish <fish@archa.cz>
 * @version 0.1
 *
 *
 */

class Viper2Router {
  /**
   * variable $instance
   * @var object instance routeru
   */
  protected static $instance = NULL;

  /**
   * variable $_uri
   * @var string aktualni uri
   */
  protected $_uri = "";
  /**
   * variable $_basePage
   * @var string aktualni basePage
   */
  protected $_basePage = "";
  /**
   * variable $_userFolder
   * @var string aktualni userFolder
   */
  protected $_userFolder = "";
  /**
   * variable $routes
   * @var array routovaci pole
   */
  protected $routes = array();
  /**
   * variable $routesReverse
   * @var array obracene routovaci pole pro generovani odkazu
   */
  protected $routesReverse = array();
  /**
   * static $localizeLinks
   * @static boolean prekladat uri
   */
  protected $localizeLinks = FALSE;
  /**
   * variable $lowercaseLinks
   * @var boolean prevadet url na male znaky
   */
  public $lowercaseLinks = TRUE;
  /**
   * variable $cacheRoutes
   * @var boolean udrzovat runtime cache pro reverseRoute ?
   */
  public $cacheRoutes = TRUE;
  /**
   * variable $routeCache
   * @var array runtime cache routovani
   */
  protected $routeCache = array();
  /**
   * variable $matchMissingSlash
   * @var boolean kontrolovat link na chybejici zaverecne /
   */
  public $matchMissingSlash = TRUE;
  
  public $useOfflineRouteCache = TRUE;
  
  protected $offlineRoutes = array();

  
  
  public function uri() {
    return($this->_uri);
  }
  public function basePage() {
    return($this->_basePage);
/*    
    if ($this->userFolder()=="") {
      return("");
    } else {
      return($this->_basePage);
    }
*/    
  }
  public function userFolder() {
    return($this->_userFolder);
  }
  
  /**
   * variable $targetPlugin
   * @var string cilovy plugin pro zpracovani
   */
  protected $targetPlugin = "Default";

  /**
   * variable $targetAction
   * @var string cilova akce pluginu
   */
  protected $targetAction = "default";

  /**
   * variable $targetLayout
   * @var string cilovy layout
   */
  protected $targetLayout = "default";
  
  /**
   * variable $targetParams
   * @var array parametry pro cilovou funkci
   */
  protected $targetParams = array();
  
  public function getInstance() {
    if (self::$instance === NULL) {
      self::$instance = new Viper2Router;
    }
    return(self::$instance);
  }
  
  // {{{ normalizeName
  /**
   * prevede text na SEO format url
   *
   * @param string $str text k prevodu
   * @return string
   */
  public function normalizeName($str,$encoding="utf-8"){
    if (is_int($str)) {
      // echo "int=".($str+0)._nl;
      return($str);
    } else {
      $str=iconv($encoding, "ASCII//TRANSLIT",$str);
      $str=preg_replace("/[^a-zA-Z0-9]+/","-",$str);
      // $str=preg_replace("/\-+/","-",$str);
      $str=preg_replace("/\-$/","",$str);
      $str=preg_replace("/^\-/","",$str);
      return($str);
    }
  }
  // }}} normalizeName

  // {{{ redirectTo
  /**
   * ukonci stranku a presmeruje na jinou
   *
   * @param string $url
   * @param boolean $perm
   * @return void
   */
  public function redirectTo($url, $permanent=FALSE){
    if (isset($_SESSION)) {
      session_write_close();
    }
    if ($permanent) {
      header("HTTP/1.1 301 Moved Permanently");
    }
    // oddelim get parametry, pokud v linku jsou
    $urlSplit = explode("?", $url, 2);
    
    if (preg_match("/^.+::.+$/", $urlSplit[0])) {
      $targetUrl = $this->basePage().$this->makeLink(array("to"=>$urlSplit[0]));
    } else {
      $targetUrl = $urlSplit[0];
    }
    header("Location: ".$targetUrl.(isset($urlSplit[1])?"?".$urlSplit[1]:""));
    exit;
  }
  // }}} redirectTo

  // {{{ transferTo
  /**
   * zmeni cilovy plugin a/nebo akci
   *
   * @param string $url
   * @return void
   */
  public function transferTo($url){
    $target = explode("::" ,$url);

    if ($target[0]!=""){
      if (!isset($this->routesReverse[$target[0]])) {
        throw new Exception("Trying to create link to non-existing plugin ".$target[0]);
      }
      $this->targetPlugin = $target[0]; 
    }

    if (isset($target[1]) && $target[1]!=""){
      $this->targetAction = $target[1]; 
    }
  }
  // }}} transferTo
  
  // {{{ reloadPage
  /**
   * ukonci stranku a nacte ji znovu. Pouziti pro smazani post promennych
   *
   * @return void
   */
  function reloadPage($pageUri=NULL,$transferGet=NULL) {
    $getParams="";
    if (!isset($pageUri)) {
      $pageUri=$this->uri();
    }
    if (isset($transferGet)) {
      if(is_bool($transferGet) && $transferGet){
        $transferGet=array_keys($_GET);
      }
      if (count($transferGet)) {
        foreach ($transferGet as $getName) {
          if (isset($_GET[$getName])) {
            $getParams.=($getParams==""?"?":"&").$getName."=".rawurlencode($_GET[$getName]);
          }
        }
      }
    }
    $this->redirectTo($this->_basePage.preg_replace("/\?[^\?]*$/","",$pageUri).$getParams);
  }
  // }}} reloadPage
  
  private function __construct() {
    define("_IS_SSL",(isset($_SERVER["HTTPS"]) && $_SERVER["HTTPS"]=="on")?1:0);

    $userFolder = "";
  
    $phpSelf=preg_replace("/^(\/[^\/]+).*$/","$1",$_SERVER["PHP_SELF"]);
    if (!preg_match("/\.php/", $phpSelf)) {
      $phpSelf = "/index.php";
    }
    $basePage=((isset($_SERVER["HTTPS"]) && $_SERVER["HTTPS"]=="on")?"https://":"http://").$_SERVER["HTTP_HOST"];
    if (preg_match("/^\/(_[^\/]+)/", $_SERVER["REQUEST_URI"],$res)) {
      $basePage .= "/".$res[1];
      $userFolder = $res[1];
    }
    if (preg_match("/^\/(sites\/[^\/]+)/", $_SERVER["REQUEST_URI"],$res)) {
      $basePage .= "/".$res[1];
      $userFolder = $res[1];
    }
    
    $uri = $_SERVER["REQUEST_URI"];
    $uri = preg_replace("/^\/_[^\/]+/","",$uri);
    $uri = preg_replace("/^\/sites\/[^\/]+/","",$uri);
    if (!defined("DEFAULT_LANGUAGE")) {
      define("_LANGUAGE",'cs');
    }
    if (preg_match("/^\/(\w{2})\/"."/",$uri,$res)) {
      // nejaka jazykova verze, zkontrolujeme, jestli je podporovana
      if (function_exists('getSupportedLanguages')) {
        $language = $res[1];
        $uri = preg_replace("/^\/".$language."/","",$uri);
        if (defined("DEFAULT_LANGUAGE") && DEFAULT_LANGUAGE == $language) {
          // zakladni jazyk, redirect na stranku bez zakladniho jazyka
          RedirectTo($basePage.$uri,1);
        }
        $supportedLanguages = getSupportedLanguages();
        if (is_array($supportedLanguages) && array_key_exists($language, $supportedLanguages)) {
          $langParams = $SupportedLanguages[$language];
          //locales
          setlocale(LC_MONETARY, $langParams['code']);
          setlocale(LC_TIME, $langParams['code']);
          setlocale(LC_ALL, $langParams['code']);
          $basePage .= "/".$language;
          $domain = "messages";
          bindtextdomain($domain, "./locale");
          bind_textdomain_codeset($domain, 'utf-8');
          textdomain($domain);
          define("_LANGUAGE", $language);
        } else {
          $this->redirectTo("Default::404");
        }
      }
    }
    $basePage = preg_replace("/\/$/","",$basePage);
    if (!defined("_LANGUAGE")) {
      define("_LANGUAGE", DEFAULT_LANGUAGE);
    }
    
    // odstranim $_GET z $uri
    $uri = preg_replace("/\?.*$/", "", $uri);
    
    $this->_basePage = $basePage;
    $this->_uri = $uri;
    $this->_userFolder = $userFolder;
    define("_BASE_PAGE", $this->_basePage);
    define("_PAGE_URI", $this->_uri);
    define("_USER_FOLDER", $this->_userFolder);
    
    $this->setRoute("/");
  }
  
  // {{{ setRoute
  /**
   * nastaveni routovaci uri
   *
   * @param string $link regularni vyraz uri
   * @param string $plugin cilovy plugin
   * @param string $action akce pluginu
   * @param string $layout cilovy layout
   *
   * @return void
   */
  public function setRoute($link, $plugin="Default", $action="default", $layout="default", $mask="", $params=array(), $addShortLink=FALSE, $addVeryShortLink=FALSE, $linkDirection="both") {
    // pokud nektera z fazi meni layout, uprednostnim ho oproti defaultu
    if (isset($this->routes[$link]["layout"]) && $this->routes[$link]["layout"]!="default") {
      $layout = $this->routes[$link]["layout"];
    }
    if ($linkDirection!="out") {
      $this->routes[$link] = array("layout"  =>  $layout,
                                   "plugin"  =>  $plugin,
                                   "action"  =>  $action,
                                   "mask"    =>  $mask,
                                   "params"  =>  $params,
                                   
                                   );
    }
    if ($linkDirection!="in") {
      $this->routesReverse[$plugin][$action][$link] = array(
                                                            "mask"    =>  $mask,
                                                            "params"  =>  $params,
                                                            );
    }
    
    if ($addShortLink && preg_match("/\/index.htm$/", $link)) {
      // pokud je povolene generovani zkracenych url, pridam variantu / -> /index.htm
      $shortMask = preg_replace("/\/index.htm$/", "/", $mask);
      $shortLink = preg_replace("/\/index.htm$/", "/", $link);

      if ($linkDirection!="out") {
        $this->routes[$shortLink] = array("layout"  =>  $layout,
                                     "plugin"  =>  $plugin,
                                     "action"  =>  $action,
                                     "mask"    =>  $shortMask,
                                     "params"  =>  $params,
                                     
                                     );
      }
      if ($linkDirection!="in") {
        $this->routesReverse[$plugin][$action][$shortLink] = array(
                                                              "mask"    =>  $shortMask,
                                                              "params"  =>  $params,
                                                              );
      }

    }
    if ($addVeryShortLink && preg_match("/\/index.htm$/", $link)) {
      // @deprecated: zkracene url bez zaverecneho /
      $shortMask2 = preg_replace("/\/index.htm$/", "", $mask);
      $shortLink2 = preg_replace("/\/index.htm$/", "", $link);
      if ($shortLink2!="") {
        if ($linkDirection!="out") {
          $this->routes[$shortLink2] = array("layout"  =>  $layout,
                                       "plugin"  =>  $plugin,
                                       "action"  =>  $action,
                                       "mask"    =>  $shortMask2,
                                       "params"  =>  $params,
                                       
                                       );
        }
        if ($linkDirection!="in") {
          $this->routesReverse[$plugin][$action][$shortLink2] = array(
                                                                "mask"    =>  $shortMask2,
                                                                "params"  =>  $params,
                                                                );
        }
      }
    }
  }
  // }}} setRoute
  
  // {{{ processRoutes
  /**
   * vyhodnoceni uri podle routovaci tabulky
   *
   * @todo tady se bude zpracovavat vyber jazyka
   *
   * @param string $link regularni vyraz uri
   * @param string $plugin cilovy plugin
   * @param string $action akce pluginu
   * @param string $layout cilovy layout
   *
   * @return void
   */
  public function processRoutes($uri) {
    $matchLength = 0;
    $direction = NULL;
    foreach ($this->routes as $link=>$route) {
      // echo "/".preg_quote($link,"/")."/"._nl;
      // if (preg_match("/^".preg_quote($link,"/")."$/i",$uri,$matches)) {
      // echo "/^".str_replace("/",'\/',$link)."$/i"._nl;
      if (preg_match("/^".str_replace("/",'\/',$link)."$/i", $uri, $matches) ||
          ($this->matchMissingSlash &&
           preg_match("/^".str_replace("/",'\/',$link)."$/i", $uri."/", $matches)
           )
          ) {
        if (strlen($matches[0])>=$matchLength) {
          $matchLength=strlen($matches[0]);
          $direction = $route;
          // vyfiltruje z matche jen parametry akce
          $paramVals = array_intersect_key($matches,array_flip($route["params"]));
          $direction["paramVals"] = $paramVals; 
        }
      }
    }
    return($direction);
  }
  // }}} processRoutes

  // {{{ setDirection
  /**
   * nastavi cilovou akci podle uri
   *
   * @return void
   */
  public function setDirection() {
    $direction = $this->processRoutes($this->uri());
    if (!isset($direction)){
      $this->targetPlugin = "Default"; 
      $this->targetAction = "404"; 
    } else {
      $this->targetLayout = $direction["layout"]; 
      $this->targetPlugin = $direction["plugin"]; 
      $this->targetAction = $direction["action"]; 
      $this->targetParams = $direction["paramVals"]; 
    }
  }
  // }}} setDirection

  // {{{ getPlugin
  /**
   * cilovy plugin
   *
   * @return string
   */
  public function getPlugin() {
    return($this->targetPlugin);
  }
  // }}} getPlugin

  // {{{ getAction
  /**
   * cilova akce
   *
   * @return string
   */
  public function getAction() {
    return($this->targetAction);
  }
  // }}} getAction

  // {{{ getLayout
  /**
   * cilovy layout
   *
   * @return string
   */
  public function getLayout() {
    return($this->targetLayout);
  }
  // }}} getLayout

  // {{{ getParams
  /**
   * cilove parametry
   *
   * @return array
   */
  public function getParams() {
    return($this->targetParams);
  }
  // }}} getParams
  
  // {{{ getParam
  /**
   * vrati obsah jednoho parametru pokud existuje
   *
   * @return mixed
   */
  public function getParam($param) {
    return(isset($this->targetParams[$param]) ? $this->targetParams[$param] : NULL);
  }
  // }}} getParam

  public function __toString() {
    return("<pre>routesReverse = ".htmlspecialchars(print_r($this->routesReverse, TRUE))."<br />routes = ".htmlspecialchars(print_r($this->routes, TRUE))."</pre>");
  }
  
  // {{{ buildLink
  /**
   * vytvoreni linku z uri a akce
   *
   * @param string $plugin cilovy plugin
   * @param string $action akce pluginu
   * @param string $layout cilovy layout
   *
   * @return string
   */
  public function buildLink($uri, $action="default", $mask="") {
    $lUri = $this->localizeLinks?_($uri):$uri;
    if ($mask!=""){
      $link = $lUri.(!preg_match("/\/$/",$lUri)?"/":"").preg_replace("/^\//","",$mask);
    } else {
      if ($action=="default") {
        $link[] = $lUri.(!preg_match("/\/$/",$lUri)?"/":"")."index.htm";
        $link[] = $lUri.(!preg_match("/\/$/",$lUri)?"/":"");
      } else {
        if (strpos($action, "__")!==FALSE) {
          $link = $lUri.(!preg_match("/\/$/",$lUri)?"/":"").preg_replace("/_/", "/", preg_replace("/__/", ".", $action));
        } else {
          $link = $lUri.(!preg_match("/\/$/",$lUri)?"/":"").preg_replace("/_/", "/", $action).".htm";
        }
      }
    }
    return($link);
  }
  // }}} buildLink
  
  
  // {{{ reverseRoute
  /**
   * z tabulky existujicich rout vytvori odkaz podle hledaneho cile
   *
   * @param string $plugin cilovy plugin
   * @param string $action akce pluginu
   *
   * @return string
   */
  public function reverseRoute($plugin, $action, $params) {
    $link = array();
    if (!isset($this->routesReverse[$plugin][$action])) {
      throw new Exception("Trying to resolve unknown link ".$plugin."::".$action);
    }
    
    $cacheId = $plugin."::".$action."[".join(",",array_keys($params))."]";
    
    // kontrola lokalni cache v pameti
    $mcLink = NULL;
    if ($this->cacheRoutes && isset($this->routeCache[$cacheId])) {
      $link = $this->routeCache[$cacheId];
    } else {
      if (!isset($mcLink) && $this->cacheRoutes && Viper2::hasPlugin("Memcached")) {
        $mcLink = v2("Memcached", "get", "linkMask_".$cacheId);
      }
      if (isset($mcLink)) {
        $link = $mcLink;
        $this->routeCache[$cacheId] = $link;
      } else {
        // zjisteni minimalniho poctu parametru pro danou akci
        $minimumParams = NULL;
        foreach ($this->routesReverse[$plugin][$action] as $rLink=>$linkInfo) {
          // pre_print($linkInfo);
          // if ((count($minimumParams)==0 && count($linkInfo["params"])) || count($linkInfo["params"])<count($minimumParams)) {
          if (!isset($minimumParams) || count($linkInfo["params"])<count($minimumParams)) {
            $minimumParams = $linkInfo["params"];
          }
        }
        if (!isset($minimumParams)) {
          $minimumParams = array();
        }
      
        // echo "params";
        // pre_print($params);
        // pre_print($minimumParams);
      
        if (count($minimumParams)>0 && count(array_intersect($minimumParams, array_keys($params)))<count($minimumParams)) {
          throw new Exception("Not enough parameters to build link. Minimal parameters: \"".join(", ",$minimumParams)."\", but supplied \"".join(", ",array_keys($params))."\"");
        }
  
        // pre_print($this->routesReverse[$plugin][$action]);
      
        $maximumParamCnt = 0;
        foreach ($this->routesReverse[$plugin][$action] as $rLink=>$linkInfo) {
          // jako link se nastavi ten, ke kterymu mame k dispozici nejvic parametru
          // pokud jsou parametry povinny
          if (count($minimumParams)>0 || count($params)) {
          
            $matchingParams = count(array_intersect($linkInfo["params"], array_keys($params)));
            // pouziju link pokud ma vic pouzitych parametru
            // nebo ma shodnou mnozinu parametru jako uz pouzity link 
            if($matchingParams>$maximumParamCnt || ($maximumParamCnt!=0 && $matchingParams==$maximumParamCnt && count($linkInfo["params"])==$maximumParamCnt)) {
              // echo "LINK: ".$rLink._nl;
              $maximumParamCnt = count(array_intersect($linkInfo["params"], array_keys($params)));
              $link[0] = $rLink;
            }
          
          } else {
            // pokud nejsou parametry povinny, nastavim normalni mnozinu linku
            $link[] = $rLink;
          }
        }
        if ($this->cacheRoutes) {
          $this->routeCache[$cacheId] = $link;
          if (Viper2::hasPlugin("Memcached")) {
            v2("Memcached", "set", "linkMask_".$cacheId, $link);
          }
        }
      }
    }
    
    unset($oLink);
    foreach ($link as &$oLink) {
      // pokud link obsahuje parametry,
      if (preg_match_all("/\?<([^\>]+)>/", $oLink, $match)) {
        $paramList = $match[1];
        // pre_print($params);
        // tak nahradime zastupce hodnotama parametru
        foreach ($paramList as $paramName) {
          if (substr($paramName, 0, 1)=="_") {
            // pokud je specialni nazev parametru, neprovedu upravu hodnoty
            $paramVal = isset($params[$paramName])?$params[$paramName]:"";
          } else {
            $paramVal = $this->normalizeName(isset($params[$paramName])?$params[$paramName]:"");
          }
          $oLink = preg_replace("/\(\?\<".$paramName."\>[^\)]+\)/", $paramVal, $oLink);
        }
      }
    }
    unset($oLink);
    
    // pre_print($link);
    return($link);
  }
  // }}} reverseRoute
  
  public function makeLink($params) {
    if (!isset($params["to"])) {
      $params["to"] = 'Default::default';
    }
    if (preg_match("/^(\w+::[^\(]+)(?<params>\(.+\))$/", $params["to"], $match)) {
      $params["to"] = $match[1];
      // ochrana proti uvozovkam v parametrech (c) Robert
      $escapedParams = str_replace("\\\"", chr(1), $match["params"]);
      $escapedParams = str_replace("\\'", chr(2), $escapedParams);
      // echo $escapedParams."\n";
      if (preg_match_all("/(?<paramName>\w+)=[\"'](?<paramVal>[^\"']+)[\"']/", $escapedParams, $paramMatch)) {
        // pre_print($paramMatch);
        $extraParams = array();
        foreach ($paramMatch["paramName"] as $paramId=>$paramName) {
          $unescapedParam = str_replace(chr(1), "\"", $paramMatch["paramVal"][$paramId]);
          $unescapedParam = str_replace(chr(2), "'", $unescapedParam);
          // echo $unescapedParam."\n";
          $extraParams[$paramName] = $unescapedParam; 
        }
        $params = array_merge($params, $extraParams);
      }
    }
    $pluginInfo = explode("::",$params["to"]);
    unset($params["to"]);
    if (!isset($pluginInfo[1])) {
      $pluginInfo[1] = "default";
    }
    $pluginName = $pluginInfo[0];
    $action = $pluginInfo[1];

    if (!isset($this->routesReverse[$pluginName])) {
      throw new Exception("Trying to create link to non-existing plugin ".$pluginName);
    }
    // if (!method_exists(self::$activePlugins[$pluginName],"show".$action) && !method_exists(self::$activePlugins[$pluginName],"action".$action) && !method_exists(self::$activePlugins[$pluginName],"render".$action)) {
    //   throw new Exception("Trying to create link to non-existing plugin's action ".$pluginName."->".$action);
    // }
    
    // echo "makeLink"._nl;
    $link = NULL;
    
    $autoLink = $this->reverseRoute($pluginName, $action, $params);
    if (is_array($autoLink)) {
      foreach ($autoLink as $subLink) {
        if(!isset($link) || strlen($subLink)<strlen($link)){
          $link = $subLink;
        }
      }
    } else {
      $link = $autoLink;
    }
    
    if ($this->lowercaseLinks) {
      $link = strtolower($link);
    }

    return($link);
  }
  
   
}
